// C Header File
// Created 8/10/2003; 7:03:46 PM

//custom gray sprite functions

//working buffers
void *light_vscreen=0, *dark_vscreen=0;

//does some initilization to create buffers and return FALSE in case of error
int GraySpriteInit(unsigned long size_buff) {
	light_vscreen = malloc(size_buff);
	if(!light_vscreen) return -1;
	dark_vscreen = malloc(size_buff);
	if(!dark_vscreen) {free(light_vscreen); return -1;}
	memset(light_vscreen, 0x00, size_buff);
	memset(dark_vscreen, 0x00, size_buff);
	GrayOn();
	if(!IsGrayMode()) {free(light_vscreen); free(dark_vscreen); GrayOff(); return -1;}
	return 1;
}

//cleans up/deinitilizes gray sprites
int GraySpriteDeInit(void) {
	free(light_vscreen);
	free(dark_vscreen);
	GrayOff();
	return 1;
}

//the CSprite functions are modified forms of the Sprite functions in sprites.h
//I've changed them so that they will be clipped to the rectangle defined by
//the screen of any 68K calculator.  the modifications make it so that any sprite
//of any mode can be drawn to any buffer at any point and not cause an address
//error (in other words, they are "safe" routines)

void CSprite8(short x, short y, short h, unsigned char *sprite, void *buff, short mode)
{
  long addr=(long)buff+30*y+((x>>3)&0xfffe),d1;
  unsigned short cnt=24-(x&15),data;
  short tempy=y;
  for(;h--;addr+=30)
    {
      data=*sprite++;
      if(x<0&&mode!=SPRT_AND) {data<<=abs(x); data>>=abs(x);} //clip the left side of the sprite because it is overlapping the left side of the screen
      else if(x<0&&mode==SPRT_AND) {
      	data=~data; 										//because AND changes bits set to 0, we have to
      	data<<=abs(x); data>>=abs(x);		//negate the data before we clip it, and then 
      	data=~data;											//negate it again to get the appropriate result
      }
      if(x>LCD_WIDTH-8&&mode!=SPRT_AND) {data>>=x-(LCD_WIDTH-8); data<<=x-(LCD_WIDTH-8);} //clip the right side of the sprite because it is overlapping the right side of the screen
      else if(x>LCD_WIDTH-8&&mode==SPRT_AND) {
      	data=~data;
      	data>>=x-(LCD_WIDTH-8); data<<=x-(LCD_WIDTH-8); 
      	data=~data;
      }
      tempy++;
      if(mode!=SPRT_AND&&(tempy<0||tempy>LCD_HEIGHT)) data=0;
      else if(mode==SPRT_AND&&(tempy<0||tempy>LCD_HEIGHT)) data=0b11111111;
      if(mode==SPRT_AND&&tempy>=0&&tempy<=LCD_HEIGHT) *(long*)addr&=~((long)(~data&0xff)<<cnt);
      else
        {
          d1=(long)data<<cnt;
          if(mode==SPRT_XOR&&tempy>=0&&tempy<=LCD_HEIGHT) *(long*)addr^=d1;
          else if(tempy>=0&&tempy<=LCD_HEIGHT) *(long*)addr|=d1;
        }
    }
}

void CSprite16(short x, short y, short h, unsigned short *sprite, void *buff, short mode)
{
  long addr=(long)buff+30*y+((x>>3)&0xfffe),d1;
  unsigned short cnt=16-(x&15),data;
  short tempy=y;
  for(;h--;addr+=30)
    {
      data=*sprite++;
      if(x<0&&mode!=SPRT_AND) {data<<=abs(x); data>>=abs(x);} //clip the left side of the sprite because it is overlapping the left side of the screen
      else if(x<0&&mode==SPRT_AND) {
      	data=~data; 										//because AND changes bits set to 0, we have to
      	data<<=abs(x); data>>=abs(x);		//negate the data before we clip it, and then 
      	data=~data;											//negate it again to get the appropriate result
      }
      if(x>LCD_WIDTH-16&&mode!=SPRT_AND) {data>>=x-(LCD_WIDTH-16); data<<=x-(LCD_WIDTH-16);} //clip the right side of the sprite because it is overlapping the right side of the screen
      else if(x>LCD_WIDTH-16&&mode==SPRT_AND) {
      	data=~data;
      	data>>=x-(LCD_WIDTH-16); data<<=x-(LCD_WIDTH-16); 
      	data=~data;
      }
      tempy++;
      if(mode!=SPRT_AND&&(tempy<0||tempy>LCD_HEIGHT)) data=0;
      else if(mode==SPRT_AND&&(tempy<0||tempy>LCD_HEIGHT)) data=0b1111111111111111;
      if(mode==SPRT_AND&&tempy>=0&&tempy<=LCD_HEIGHT) *(long*)addr&=~((long)~data<<cnt);
      else
        {
          d1=(long)data<<cnt;
          if(mode==SPRT_XOR&&tempy>0&&tempy<LCD_HEIGHT) *(long*)addr^=d1;
          else if(tempy>=0&&tempy<=LCD_HEIGHT) *(long*)addr|=d1;
        }
    }
}

void CSprite32(short x, short y, short h, unsigned long *sprite, void *buff, short mode)
{
  long addr=(long)buff+30*y+((x>>3)&0xfffe);
  unsigned long data,d1,d2;
  short cnt=x&15,ccnt=32-cnt;
  short tempy=y;
  for(;h--;addr+=30)
    {
      data=*sprite++;
      if(x<0&&mode!=SPRT_AND) {data<<=abs(x); data>>=abs(x);} //clip the left side of the sprite because it is overlapping the left side of the screen
      else if(x<0&&mode==SPRT_AND) {
      	data=~data; 										//because AND changes bits set to 0, we have to
      	data<<=abs(x); data>>=abs(x);		//negate the data before we clip it, and then 
      	data=~data;											//negate it again to get the appropriate result
      }
      if(x>LCD_WIDTH-32&&mode!=SPRT_AND) {data>>=x-(LCD_WIDTH-32); data<<=x-(LCD_WIDTH-32);} //clip the right side of the sprite because it is overlapping the right side of the screen
      else if(x>LCD_WIDTH-32&&mode==SPRT_AND) {
      	data=~data;
      	data>>=x-(LCD_WIDTH-32); data<<=x-(LCD_WIDTH-32); 
      	data=~data;
      }
      tempy++;
      if(mode!=SPRT_AND&&(tempy<0||tempy>LCD_HEIGHT-0)) data=0;
      else if(mode==SPRT_AND&&(tempy<0||tempy>LCD_HEIGHT-0)) data=0b11111111111111111111111111111111;
      if(mode==SPRT_AND)
        {
          data=~data;
          if(tempy>=0&&tempy<=LCD_HEIGHT-0)*(long*)addr&=~(data>>cnt),*(long*)(addr+4)&=~(data<<ccnt);
        }
      else
        {
          d1=data>>cnt; d2=data<<ccnt;
          if(mode==SPRT_XOR&&tempy>=0&&tempy<=LCD_HEIGHT-0) *(long*)addr^=d1,*(long*)(addr+4)^=d2;
          else if(tempy>=0&&tempy<=LCD_HEIGHT-0) *(long*)addr|=d1,*(long*)(addr+4)|=d2;
        }
    }
}

//shorthand functions that use CSprite in grayscale mode
void GraySprite8 (short x, short y, short h, char *spr, void *foo, short mode)
{
	if(IsGrayMode()) {
		CSprite8 (x, y, h, spr, light_vscreen, mode);
		CSprite8 (x, y, h, spr, dark_vscreen, mode);
		CSprite8 (x, y, h, spr+h, light_vscreen, mode);
	}
	else CSprite8 (x, y, h, spr, foo, mode);
}

void GraySprite16 (short x, short y, short h, short *spr, void *foo, short mode)
{
	if(IsGrayMode()) {
		CSprite16 (x, y, h, spr, light_vscreen, mode);
		CSprite16 (x, y, h, spr, dark_vscreen, mode);
		CSprite16 (x, y, h, spr+h, light_vscreen, mode);
	}
	else CSprite16 (x, y, h, spr, foo, mode);
}

void GraySprite32 (short x, short y, short h, long *spr, void *foo, short mode)
{
	if(IsGrayMode()) {
		CSprite32 (x, y, h, spr, light_vscreen, mode);
		CSprite32 (x, y, h, spr, dark_vscreen, mode);
		CSprite32 (x, y, h, spr+h, light_vscreen, mode);
	}
	else CSprite32 (x, y, h, spr, foo, mode);
}

//draws a clipped line-clipped to screen boundries.  Faster than DrawClipLine()
void CDrawLine(short x1, short y1, short x2, short y2, void *buff, short mode)
{
  short x = x1, y = y1;
  short dx = abs (x2 - x1), dy = abs (y2 - y1);
  short ystep = (y1 < y2) ? 1 : -1, pystep = 30 * ystep;
  short mov = dx ? 0 : -1;
  unsigned char *ptr = (char*)buff + 30 * y + (x >> 3);
  short mask = 1 << (~x & 7);
  if (x1 < x2)
    while (x != x2 || y != y2)
      {
        if(mode==A_OR&&x>=10&&y>=10&&x<=LCD_WIDTH-10&&y<=LCD_HEIGHT-10) *ptr |= mask;
        if(mode==A_XOR&&x>=10&&y>=10&&x<=LCD_WIDTH-10&&y<=LCD_HEIGHT-10) *ptr ^= mask;
        if(mode==A_AND&&x>=10&&y>=10&&x<=LCD_WIDTH-10&&y<=LCD_HEIGHT-10) *ptr &= mask;
        if (mov < 0) y += ystep, ptr += pystep, mov += dx;
        else
          {
            mov -= dy;
            if (++x & 7) mask >>= 1;
            else ptr++, mask = 0x80;
          }
      }
  else
    while (x != x2 || y != y2)
      {
        if(mode==A_OR&&x>=10&&y>=10&&x<=LCD_WIDTH-10&&y<=LCD_HEIGHT-10) *ptr |= mask;
        if(mode==A_XOR&&x>=10&&y>=10&&x<=LCD_WIDTH-10&&y<=LCD_HEIGHT-10) *ptr ^= mask;
        if(mode==A_AND&&x>=10&&y>=10&&x<=LCD_WIDTH-10&&y<=LCD_HEIGHT-10) *ptr &= mask;
        if (mov < 0) y += ystep, ptr += pystep, mov += dx;
        else
          {
            mov -= dy;
            if (x-- & 7) mask <<= 1;
            else ptr--, mask = 1;
          }
      }
}

//copies working buffers to gray planes.  Requires ExtGraph
void buffer_to_screen(void) {
	FastCopyScreen(light_vscreen, GetPlane(LIGHT_PLANE));
	FastCopyScreen(light_vscreen, GetPlane(DARK_PLANE));
	FastCopyScreen(dark_vscreen, GetPlane(DARK_PLANE));
}

